﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2013 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Operators;
using HeuristicLab.Optimization;
using HeuristicLab.Parameters;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Analysis.SolutionCaching {
  [StorableClass]
  public abstract class SolutionCacheAnalyzer<TKey, TValue> : SingleSuccessorOperator, ISolutionCacheAnalyzer
    where TKey : Item
    where TValue : SolutionInformation<TKey> {
    public ValueLookupParameter<ResultCollection> ResultsParameter {
      get { return (ValueLookupParameter<ResultCollection>)Parameters["Results"]; }
    }

    public ValueLookupParameter<SolutionCache<TKey, TValue>> SolutionCacheParameter {
      get { return (ValueLookupParameter<SolutionCache<TKey, TValue>>)Parameters["SolutionCache"]; }
    }

    [StorableConstructor]
    protected SolutionCacheAnalyzer(bool deserializing) : base(deserializing) { }
    protected SolutionCacheAnalyzer(SolutionCacheAnalyzer<TKey, TValue> original, Cloner cloner)
      : base(original, cloner) {
    }

    public SolutionCacheAnalyzer()
      : base() {
      Parameters.Add(new ValueLookupParameter<ResultCollection>("Results", "The variable collection where results should be stored."));
      Parameters.Add(new ValueLookupParameter<SolutionCache<TKey, TValue>>("SolutionCache", "The solution cache."));
    }

    public abstract void ExecuteAnalysis(ResultCollection results, SolutionCache<TKey, TValue> solutionCache);

    public override IOperation Apply() {
      ExecuteAnalysis(ResultsParameter.ActualValue, SolutionCacheParameter.ActualValue);
      return base.Apply();
    }
  }
}
